"""CLI."""

from enum import StrEnum
from pydantic import BaseModel, Field, ConfigDict
from .._core import User, Workspace, GraphQLClient  # pylint: disable=no-name-in-module


class Commands(StrEnum):
    """Commands."""

    EXIT = "exit"


class CLIState(BaseModel):
    """Holds the CLI state."""

    model_config = ConfigDict(validate_default=False, arbitrary_types_allowed=True)

    user: User
    workspace: Workspace = Field(default=None)
    client: GraphQLClient



"""Header file for ptolemy core."""

# pylint: disable=unused-argument,missing-function-docstring,too-few-public-methods
from __future__ import annotations
from typing import Optional, Any, Dict, TypeVar, List, Literal, Iterable
from uuid import UUID
from datetime import datetime
from enum import Enum
import pandas as pd

T = TypeVar("T", bound=Enum)

class ApiKeyPermission(Enum):
    """API Key Permissions Enum."""

    READ_ONLY: str = "READ_ONLY"
    WRITE_ONLY: str = "WRITE_ONLY"
    READ_WRITE: str = "READ_WRITE"

class UserStatus(Enum):
    """User Status Enum."""

    ACTIVE: str = "ACTIVE"
    SUSPENDED: str = "SUSPENDED"

class WorkspaceRole(Enum):
    """Workspace Role Enum."""

    USER: str = "USER"
    MANAGER: str = "MANAGER"
    ADMIN: str = "ADMIN"

class Ptolemy:
    """Ptolemy Client."""

    def __init__(
        self,
        base_url: str,
        api_key: str,
        workspace_name: str,
        autoflush: bool,
        batch_size: int,
    ) -> "Ptolemy": ...
    def trace(
        self,
        name: str,
        parameters: Optional[Dict[str, Any]] = None,
        version: Optional[str] = None,
        environment: Optional[str] = None,
    ) -> "Ptolemy": ...
    def child(
        self,
        name: str,
        parameters: Optional[Dict[str, Any]] = None,
        version: Optional[str] = None,
        environment: Optional[str] = None,
    ) -> "Ptolemy": ...
    def event(
        self,
        name: str,
        parameters: Optional[Dict[str, Any]] = None,
        version: Optional[str] = None,
        environment: Optional[str] = None,
    ) -> None: ...
    def runtime(
        self,
        start_time: float,
        end_time: float,
        error_type: Optional[str] = None,
        error_content: Optional[str] = None,
    ) -> None: ...
    def inputs(self, **kwds: Any) -> None: ...
    def outputs(self, **kwds: Any) -> None: ...
    def feedback(self, **kwds: Any) -> None: ...
    def metadata(self, **kwds: Any) -> None: ...
    def push_event(self) -> bool: ...
    def push_io(self) -> bool: ...
    def flush(self) -> bool: ...

    def sql(self, query: str) -> Iterable[pd.DataFrame]: ...

class Workspace:
    """Workspace object."""

    id: UUID
    name: str
    description: Optional[str]
    archived: bool
    created_at: datetime
    updated_at: datetime

    def to_dict(self) -> dict: ...

class User:
    """User object."""

    id: UUID
    username: str
    display_name: Optional[str]
    status: UserStatus
    is_admin: bool
    is_sysadmin: bool

    def to_dict(self) -> dict: ...

class UserApiKey:
    """User API key."""

    id: UUID
    user_id: UUID
    name: str
    key_preview: str
    expires_at: Optional[datetime]

    def to_dict(self) -> dict: ...

class ServiceApiKey:
    """Service API key."""

    id: UUID
    workspace_id: UUID
    name: str
    key_preview: str
    permissions: ApiKeyPermission
    expires_at: Optional[datetime]

    def to_dict(self) -> dict: ...

class WorkspaceUser:
    """Workspace user."""

    workspace_id: UUID
    user_id: UUID
    role: WorkspaceRole

    def to_dict(self) -> dict: ...

class GraphQLClient:
    """GraphQL client."""

    def __init__(
        self,
        url:str ,
        api_key: str,
        auth_method: Literal['api_key', 'jwt'] = 'api_key'
        ): ...
    def me(self) -> User: ...
    def create_workspace(
        self,
        name: str,
        admin_user_id: UUID,
        description: Optional[str] = None,
    ) -> Workspace: ...
    def delete_workspace(
        self,
        workspace_id: UUID,
    ): ...
    def add_user_to_workspace(
        self,
        target_user_id: UUID,
        workspace_id: UUID,
        role: WorkspaceRole,
    ): ...
    def remove_user_from_workspace(self, workspace_id: UUID, user_id: UUID): ...
    def change_user_workspace_role(
        self,
        user_id: UUID,
        workspace_id: UUID,
        role: WorkspaceRole,
    ): ...
    def create_service_api_key(
        self,
        workspace_id: UUID,
        name: str,
        permissions: ApiKeyPermission,
        valid_for: Optional[int] = None,
    ) -> str: ...
    def delete_service_api_key(self, workspace_id: UUID, api_key_id: UUID): ...
    def get_workspace_service_api_keys(
        self, workspace_id: UUID
    ) -> List[ServiceApiKey]: ...
    def get_user_workspace_role(
        self, workspace_id: UUID, user_id: UUID
    ) -> WorkspaceRole: ...
    def get_workspace_users_by_name(
        self, workspace_name: str
    ) -> List[tuple[WorkspaceRole, User]]: ...
    def get_workspace_users(
        self, workspace_id: UUID
    ) -> List[tuple[WorkspaceRole, User]]: ...
    def create_user(
        self,
        username: str,
        password: str,
        is_admin: bool,
        is_sysadmin: bool,
        display_name: Optional[str] = None,
    ) -> User: ...
    def delete_user(self, user_id: UUID): ...
    def create_user_api_key(
        self, name: str, duration_days: Optional[int] = None
    ) -> str: ...
    def delete_user_api_key(self, api_key_id: UUID): ...
    def all_users(self) -> List[User]: ...
    def get_user_by_name(self, username: str) -> User: ...
    def get_user_workspaces(self, user_id: UUID) -> List[Workspace]: ...
    def get_user_workspaces_by_username(
        self, username: str
    ) -> List[tuple[WorkspaceRole, Workspace]]: ...
    def get_user_api_keys(self, user_id: UUID) -> List[UserApiKey]: ...



"""CLI."""

# pylint: disable=wildcard-import,unused-wildcard-import
import shlex
import click
from prompt_toolkit import PromptSession
from prompt_toolkit.completion import WordCompleter
from .cli.login import select_workspace
from .cli import get_cli
from .cli.cli import CLIState, Commands
from . import GraphQLClient


def run_cli():
    """Run Ptolemy CLI."""
    session = PromptSession()
    completer = WordCompleter(list(Commands))
    client = None
    current_user = None

    while client is None and current_user is None:
        key = session.prompt("Please enter your API key: > ")
        client = GraphQLClient("http://localhost:8000/external/graphql", key)
        try:
            current_user = client.me()
        except ValueError as e:
            click.echo(f"Failed to login. Please try again. Details: {e}")
            continue

        click.echo(f"Welcome, {current_user.username}! 💚")
        cli_data = {"user": current_user, "client": client}
        wk = select_workspace(current_user, client)

        if wk:
            cli_data["workspace"] = wk

        cli_state = CLIState(**cli_data)

    while True:
        cmd = session.prompt("💚 ptolemy> ", completer=completer, is_password=False)

        if cmd == Commands.EXIT:
            break

        # Parse and execute command
        args = shlex.split(cmd)
        try:
            cli = get_cli(cli_state.user)
            # Pass the CLI state through the context
            ctx = click.Context(cli)
            ctx.obj = {"state": cli_state}
            cli.main(args, prog_name="Ptolemy", standalone_mode=False, obj=ctx.obj)
        except click.exceptions.UsageError:
            # Show command-specific help when usage is wrong
            command_name = args[0] if args else ""
            if command := cli.get_command(None, command_name):
                ctx = click.Context(command)
                click.echo(command.get_help(ctx))


if __name__ == "__main__":
    run_cli()



"""CLI."""

import click
from .._core import User  # pylint: disable=no-name-in-module
from .workspace import workspace
from .user import user


def get_cli(usr: User):
    """Get CLI function."""

    @click.group()
    @click.pass_context
    def cli(ctx):
        """CLI root command group."""
        # Ensure ctx.obj exists
        ctx.ensure_object(dict)

    if not usr.is_sysadmin:
        cli.add_command(workspace, name="workspace")

    cli.add_command(user, name="user")

    return cli



"""Login."""

from typing import Optional
import questionary
from .._core import GraphQLClient, User, Workspace  # pylint: disable=no-name-in-module


def select_workspace(usr: User, client: GraphQLClient) -> Optional[Workspace]:
    """Select workspaces."""
    workspaces = {wk.name: wk for wk in client.get_user_workspaces(usr.id)}

    if workspaces:
        wk = questionary.select(
            "Select a workspace:",
            choices=workspaces,
            use_shortcuts=True,
        ).ask()

        return workspaces[wk]

    return None



"""Format utils."""

import click
from .._core import User  # pylint: disable=no-name-in-module


def format_user_info(user: User):
    """Format user info."""
    return (
        f"{click.style('ID:', bold=True)} {click.style(user.id, fg='blue')}\n"
        f"{click.style('Username:', bold=True)} {click.style(user.username, fg='cyan')}\n"
        f"{click.style('Is Admin:', bold=True)} {click.style(str(user.is_admin), fg='green' if user.is_admin else 'red')}\n"
        f"{click.style('Is Sysadmin:', bold=True)} {click.style(str(user.is_sysadmin), fg='green' if user.is_sysadmin else 'red')}\n"
        f"{click.style('Display Name:', bold=True)} {click.style(user.display_name, fg='cyan')}\n"
        f"{click.style('Status:', bold=True)} {click.style(user.status, fg='yellow')}"
    )
